
 !=============================================================================================
 ! NDACC_ResolDF_idl
 !
 ! Purpose: idl interface for NDACC_ResolDF (debug tool)
 !=============================================================================================

 SUBROUTINE NDACC_ResolDF_idl(argc, argv)    !Called by IDL  
 integer, intent(in)                   :: argc
 integer, intent(in), dimension(argc)  :: argv

 j = LOC(argc)         !Obtains the number of arguments (argc)  
                       !Because argc is passed by VALUE. 

 !Call subroutine NDACC_ResolDF, converting the IDL parameters  
 !to standard Fortran, passed by reference arguments:    
 CALL NDACC_ResolDF(%VAL(argv(1)), %VAL(argv(2)), %VAL(argv(3)), %VAL(argv(4)), %VAL(argv(5)), %VAL(argv(6)), %VAL(argv(7)), %VAL(argv(8)))  

 RETURN  
 END SUBROUTINE NDACC_ResolDF_idl


 ! BELOW ARE SUGGESTED FINAL COMMENTS FOR DISTRIBUTION (PLEASE CHECK AND REVISE AS APPROPRIATE)
 !This routine computes vertical resolution using the ISSI-Team-agreed standardized defintion based on the cutoff frequency of digitial filters.
 !For details, please refer to ISSI Team Report at http://www.issibern.ch/teams/ndacc/ISSI_Team_Report.htm
 !IMPORTANT: If several filtering events occur during data processing, this routine must be called each time a filtering event occurs.
 !           The values of "hfout" returned during the first call must be used for "hfinp" in the second call, and so on until the last call.
 !           The returned vertical resolution "dzcutf" of the last call only needs to be used (no need for those computed during previosu calls)

 !File history:
 !Created in 2011 by Thierry Leblanc, NASA-Jet Propulsion Laboratory, thierry.leblanc@jpl.nasa.gov
 !Revised in June 2015 by T. Leblanc: Introduce factor of 2 ( i.e., 1/(2*fc) instead of 1/fc ) to add consistency with impulse response definition
 !Revised in Nov. 2015 by T. Leblanc: Introduce input parameter "dzsampling" and use it to compute vertical resolution in physical unit (last line of routine)

 !Inputs:
 !    dzsampling = Vertical sampling resolution of signal/profile being filtered (in physical unit)
 !    coef       = A vector of 2*N+1 coefficients containing the values of the filter coefficients used to smooth or differentiate the signal/profile
 !                 The coefficients must be either symmetric (c(k)=c(-k) for all k), or anti-symmteric (c(k)=-c(-k) for all k)
 !    hfinp      = A vector (suggested length = total length of your signal/profile) containing the values of the transfer function in the frequency domain.
 !                 If routine first call, it is equal to 1 for all frequencies
 !                 If not first call, it must be equal to the transfer function "hfout" returned by the routine during the previous call
 !Outputs:
 !    status = Informaiton flag on computaiton status: 1 = succes (smoothing) , 2 = sucess (differentiation) , 0 = failure (cancel)
 !    dzcutf = The value of vertical resolution using the standardized defintion based on the FWHM of an impulse response
 !             (same unit as the input sampling resolution)
 !    hfout  = A vector of same lenght as hfinp containing the values of the calculated transfer function used to compute the cutoff frequency.
 !             If this routine is called again later, use this output as the input transfer function "hfinp" for the next call


 ! ORIGINAL COMMENTS BY FRANCK? OR GUILLAUME?
 !==============================================================================================
 ! NDACC_ResolDF
 !
 ! Purpose: calculation of the normalized resolution of the filter with the digital filter method.
 ! To obtain the real resolution, multiplie dzcutf output value by the initial resolution.
 ! Desired filter is defined by input coefficients vector coef.
 ! You can use NDACC_ResolDF as a multi-call function ; in this case you can give in input 
 ! an existing transfer function (hfinp). If you don't want to use a pre-existing transfer function
 ! set hfinp with 1 values.   
 !
 ! status: 
 !   - 0: problem
 !   - 1: low-pass filter
 !   - 2: derivative filter
 !==============================================================================================

 SUBROUTINE NDACC_ResolDF ( dzsampling, ncoef, coef, nf, hfinp, status, dzcutf, hfout)

 IMPLICIT NONE

 !--Parameters declaration
 !------------------------
 !--Inputs/Outputs
 real :: dzsampling                                  !sampling resolution (in physical unit)
 integer, intent(in) :: ncoef                        !number of filter's coefficients (size of coef)
 real (kind=8), dimension(ncoef), intent(in) :: coef !coefficients of the filter
 integer, intent(in) :: nf                           !size of the transfer functions
 real (kind=8), dimension(nf), intent(in) :: hfinp   !transfer function of the input filter (set to [1...1] by default)
 integer, intent(out) :: status                      !output status (0:error, 1:smoothing filter, 2:derivative filter)
 real (kind=4), intent(out) :: dzcutf                !vertical resolution in same unit as dzsampling
 real (kind=8), dimension(nf), intent(out) :: hfout  !transfer function of the output filter (set to [1...1] by default)
 !--Parameters
 real (kind=8), parameter :: pi=4*datan(1d0)    !pi number
 real (kind=8), parameter :: missval=-99.       !missing value for real double precision
 real (kind=4), parameter :: missval_real=-99.  !missing value for real single precision
 real (kind=8), parameter :: eps=1.e-9          !epsilon value for real double precision
 real (kind=4), parameter :: eps_real = 1.e-5   !epsilon value for real single precision
 !--Calculation variables
 real (kind=4) :: fcinp, fc                  !frequency cut-off of the input and the output filters
 real (kind=4), dimension (nf) :: f          !frequency vector (normalized ; size=nf)
 real (kind=8), dimension (nf) :: hf         !transfer function of the filter (size=nf)
 complex (kind=8), dimension (nf) :: hfcomp  !complex transfer function of the filter (size=nf)
 integer :: i, j                             !indexes
 integer :: nc                               !filter window size -> 2*nc+1=ncoeff
 logical :: firstcall, fcfound, equalto      !control flag
 real (kind=8) :: weight, coefsum            !calculation variables

 !--Calculation variables initialization and control
 !--------------------------------------------------
 firstcall=.true.
 fcfound=.false. 
 status=0  
 weight=0.
 fc=missval_real
 fcinp=missval_real 
 dzcutf=missval_real
 if (ncoef<=0) then
   print*,"[ERROR] NDACC_ResolDF.f90: number of coefficient must greater than 0"
   RETURN
 endif
 if (nf<=0) then
   print*,"[ERROR] NDACC_ResolDF.f90: size of transfer functions must be greater than 0"
   RETURN
 endif
 do i=1,nf
   f(i)=0.5/nf + 0.5/nf*(i-1)
 enddo
 hf=missval
 hfout=missval

 !--Check if it is the subroutine first call (hfinp(*)=1) -> firstcall
 !-------------------------------------------------------------------
 do i=1,nf
   if (hfinp(i)>(1.+eps) .or. hfinp(i)<(1.-eps)) then
     firstcall=.false.
   endif
 enddo

 !--Identify frequency cut-off of the input filter -> fcinp
 !---------------------------------------------------------
 if (firstcall) then
   !--Nyquist frequency (minimum resolution resolved by lidar system)
   fcinp=0.5 
 else
   fcfound=.false.
   !--Search frequency cut-off
   do i=1,nf-1
     !frequency cut-off interval selection    
     if ((hfinp(i+1)-0.5)*(hfinp(i)-0.5) <= (0.+eps)) then
       if (abs(hfinp(i)-0.5) < eps) then
         fcinp=f(i)
       else 
         if (abs(hfinp(i+1)-0.5) < eps) then
           fcinp=f(i)
         else
           !linear interpolation between i and i+1
           weight=abs(hfinp(i)-0.5)/abs(hfinp(i+1)-hfinp(i))
           fcinp=f(i)+weight*abs(f(i+1)-f(i))                !weight = weighted distance between i and i+1
         endif
       endif
       fcfound=.true.
       exit    !stop loop
     endif
   enddo
   if (.not. fcfound) then
     fcinp=0.5
     print*,"[WARNING] NDACC_ResolDF.f90: fcinp not found, fcinp is set to 0.5 by default"
   endif
 endif

 !--coef analysis and filter's transfer function calculation
 !----------------------------------------------------------
 !--ncoef is not valid
 if  (mod(ncoef,2) == 0) then
   print*,"[ERROR] NDACC_ResolDF.f90: length of coef must be an odd value"
   RETURN
 endif
 !--Only one coef, and this coef is not equal to 1 => reject filter
 coefsum=sum(coef)
 if (ncoef==1 .and. (coefsum>(1+eps) .or. coefsum<(1-eps))) then 
   print*,"[ERROR] NDACC_ResolDF.f90: size of coef is equal to 1 but sum(coef) is not equal to 1"
   RETURN
 else
   if (ncoef==1 .and. coefsum>=(1-eps) .and. coefsum<=(1+eps)) then
     !--No filtering case
     do i=1,nf
       hf(i)=1.
       status=1 !low pass filter
     enddo
   else
     !--At least 3 coefs, and number of coefs is odd => Calculate Transfer Function Hf
     nc=(ncoef-1)/2 !number of points before and after the central point / central point = nc+1
     do i=1,nc
       if (abs(coef(nc+1-i)-coef(nc+1+i)) <= abs(coef(nc+1-i))*1e-3) then
         status = 1           !coef(nc+1-i) = coef(nc+1+i) (symmetric) =>  Smoothing filter (status=1)
       else
         equalto=.false.
         if (coef(nc+1)<=(0.+eps) .and. coef(nc+1)>=(0.-eps)) then
           equalto=.true.
         endif
         if ( ( abs(coef(nc+1-i)+coef(nc+1+i)) <= abs(coef(nc+1-i))*1e-3 ) .and. equalto ) then
           status = 2         !coef(nc+1-i) = - coef(nc+1+i) and coef(nc+1)=0 (anti-symmetric) =>  Derivative filter (status=2)
         else
           status = 0
           print*,"[ERROR] NDACC_ResolDF.f90: bad coef values: neither symmetric or anti-symmetric"
           RETURN
         endif
       endif 
     enddo
     !--Calculate hf
     do i=(nc+1),(nf-nc)
       if (status==1) then
         hf(i)=coef(nc+1)
         do j=1,nc
           hf(i)=hf(i)+2*coef(nc+1+j)*cos(2.*pi*f(i)*j)
         enddo
       else
         hf(i)=0.
         do j=1,nc
           hf(i)=hf(i)-2*coef(nc+1+j)*sin(2.*pi*f(i)*j)/(2.*pi*f(i)) !renormalize Transfer Function for derivative filters
         enddo
       endif
     enddo
     hf=abs(hf)
     !--Update hf boundaries
     do i=1,nc
       hf(i)=hf(nc+1)
     enddo
     do i=(nf-nc+1),nf
       hf(i)=hf(nf-nc)
     enddo
   endif
 endif

 !--Update Transfer Function in case of mutliple calls
 !----------------------------------------------------
 do i=1,nf
   hfout(i) = hf(i) * hfinp(i)
 enddo

 !--Identify cut-off frequency , i.e., hfout(fc)=0.5
 !--------------------------------------------------
 fcfound=.false.
 !--If only one coef, and this coef=1 => force cut-off frequency to that of the input Transfer Function
 if (ncoef == 1) then 
   fc = fcinp
 !--Search frequency cut-off
 else
   do i=1,nf-1
     !frequency cut-off interval selection    
     if ((hfout(i+1)-0.5)*(hfout(i)-0.5) <= (0+eps)) then
       if (abs(hfout(i)-0.5) < eps) then
         fc=f(i)
       else 
         if (abs(hfout(i+1)-0.5) < eps) then
           fc=f(i)
         else
           !linear interpolation between i and i+1
           weight=abs(hfout(i)-0.5)/abs(hfout(i+1)-hfout(i))
           fc=f(i)+weight*abs(f(i+1)-f(i))                !weight = weighted distance between i and i+1
         endif
       endif
       fcfound=.true.
       exit    !stop loop
     endif
   enddo
   if (.not. fcfound) then
     print*,"[WARNING] NDACC_ResolDF.f90: fc not found, fc is set to 0.5 by default"
     RETURN
   endif
 endif

 !--Calculate output resolution
 !-----------------------------
 if (fc>0.) then 
   dzcutf = dzsampling/(2*fc)
 else
   status = 0
   print*,"[ERROR] NDACC_ResolDF.f90: fc<=0: ",fc
   RETURN
 endif

 END SUBROUTINE NDACC_ResolDF

