
 !=============================================================================================
 ! NDACC_ResolIR_idl
 !
 ! Purpose: idl interface for NDACC_ResolIR (debug tool)
 !=============================================================================================

 SUBROUTINE NDACC_ResolIR_idl(argc, argv)    !Called by IDL  

 integer, intent(in)                   :: argc
 integer, intent(in), dimension(argc)  :: argv 
   
 j = LOC(argc)         !Obtains the number of arguments (argc)  
                       !Because argc is passed by VALUE.  
   
 !Call subroutine NDACC_ResolIR, converting the IDL parameters  
 !to standard Fortran, passed by reference arguments:    
 CALL NDACC_ResolIR(%VAL(argv(1)), %VAL(argv(2)), %VAL(argv(3)), %VAL(argv(4)), %VAL(argv(5)), %VAL(argv(6)), %VAL(argv(7)), %VAL(argv(8)))  

 RETURN  
 END SUBROUTINE NDACC_ResolIR_idl

 
 !BELOW ARE SUGGESTED FINAL COMMENTS FOR DISTRIBUTION (PLEASE CHECK AND REVISE AS APPROPRIATE)
 !==============================================================================================
 ! This routine computes vertical resolution using the ISSI-Team-agreed standardized defintion based on the FWHM of an Impulse Response.
 !For details, please refer to ISSI Team Report at http://www.issibern.ch/teams/ndacc/ISSI_Team_Report.htm
 !IMPORTANT: If several filtering events occur during data processing, this routine must be called each time a filtering event occurs.
 !           The values of "irout" returned during the first call must be used for "irinp" in the second call, and so on until the last call.
 !           The returned vertical resolution "dzcutf" of the last call only needs to be used (no need for vertical resolution computed during previous calls)

 !File history:
 !Created in 2011 by GUILLAUME? FRANCK? ??????? , email address?
 !Revised in Nov. 2015 by T. Leblanc: Introduce input parameter "dzsampling" and use it to compute vertical resolution in physical unit (last line of routine)

 !Inputs:
 !    dzsampling = Vertical sampling resolution of signal/profile being filtered (in physical unit)
 !    coef       = A vector of 2*N+1 coefficients containing the values of the filter coefficients used to smooth or differentiate the signal/profile
 !                 The coefficients must be either symmetric (c(k)=c(-k) for all k), or anti-symmteric (c(k)=-c(-k) for all k)
 !    irinp      = A vector (suggested length = total length of your signal/profile) containing the values of the impulse.
 !                 If routine first call, it is equal to a Kronecker delta function for smoothing filters, and a Heavyside step function for differentiators
 !                 If not first call, it must be equal to the impulse response "irout" returned by the routine during the previous call
 !Outputs:
 !    status  = Informaiton flag on computaiton status: 1 = succes (smoothing) , 2 = sucess (differentiation) , 0 = failure (cancel)
 !    dzfwhm  = The value of vertical resolution using the standardized defintion based on the FWHM of an impulse response
 !              (same unit as the input sampling resolution)
 !    irout   = A vector of same lenght as hfinp containing the values of the calculated transfer function used to compute the cutoff frequency.
 !              If this routine is called again later, use this output as the input impulse "irinp" for the next call



 ! BELOW ARE THE ORIGNAL COMMENTS BY GUILLAUME? FRANCK?
 !==============================================================================================
 ! NDACC_ResolIR
 !
 ! Purpose: calculation of the normalized resolution of the filter with the FWHM method.
 ! To obtain the real resolution, multiplie dzfwhm output value by the initial resolution.
 ! Desired filter is defined by input coefficients vector coef.
 ! You can use NDACC_ResolIR as a multi-call function ; in this case you can give in input 
 ! an existing signal (irinp). If you don't want to use a pre-existing signal
 ! set irinp with 0 values.   
 !
 ! status: 
 !   - 0: problem
 !   - 1: low-pass filter
 !   - 2: derivative filter
 !==============================================================================================

 SUBROUTINE NDACC_ResolIR ( dzsampling, ncoef, coef, nk, irinp, status, dzfwhm, irout )

 IMPLICIT NONE

 !--Parameters declaration
 !------------------------
 !--Inputs/Outputs
 real :: dzsampling                                     !sampling resolution (in physical unit)
 integer, intent(in) :: ncoef                           !number of filter's coefficients (size of coef)
 real (kind=8), dimension(ncoef), intent(in) :: coef    !coefficients of the filter
 integer, intent(in) :: nk                              !size of the input signal
 real (kind=8), dimension(nk), intent(inout) :: irinp   !additional input signal (set to [0...0] by default)
 integer, intent(out) :: status                         !output status (0:error, 1:smoothing filter, 2:derivative filter)
 real (kind=4), intent(out) :: dzfwhm                   !vertical resolution in same unit as dzsampling
 real (kind=8), dimension(nk), intent(out) :: irout     !output signal (set to [0...0] by default)
 !--Parameters
 real (kind=8), parameter :: missval=-99.       !missing value for real double precision
 real (kind=4), parameter :: missval_real=-99.  !missing value for real single precision
 real (kind=8), parameter :: eps=1.e-9          !epsilon value for real double precision
 real (kind=4), parameter :: eps_real = 1.e-5   !epsilon value for real single precision
 !--Calculation variables
 real (kind=8), dimension(nk) :: irtemp         !temporary signal
 real (kind=8) :: irmax                         !maximum value of signal
 integer :: i, k,j                              !indexes
 integer, dimension(1) :: kmax                  !signal maximum position
 real (kind=4) :: x1, x2                        !fwhm identification 
 integer :: nc                                  !filter window size -> 2*nc+1=ncoeff
 logical :: firstcall, equalto,fwhmfound        !control flag
 real (kind=8) :: weight, coefsum               !calculation variables

 !--Calculation variables initialization and control
 !--------------------------------------------------
 firstcall=.true.
 status=0         !default value = error
 weight=0.
 dzfwhm=missval_real
 if (ncoef<=0) then
   print*,"[ERROR] NDACC_ResolIR.f90: number of coefficient must be greater than 0"
   RETURN
 endif
 if (nk<=0) then
   print*,"[ERROR] NDACC_ResolIR.f90: size of signal must be greater than 0"
   RETURN
 endif
 irout=0.
 irtemp=0.

 !--Check if it is the subroutine first call (irinp(*)=0) -> firstcall
 !-------------------------------------------------------------------
 do k=1,nk
   if ( irinp(k)>(0.+eps) .or. irinp(k)<(0.-eps) ) then
     firstcall=.false.
   endif
 enddo

 !--Retrieve filter type: coef vector analysis --> status
 !-------------------------------------------------------
 !--Case of ncoef is not valid
 if  (mod(ncoef,2) == 0) then
   print*,"[ERROR] NDACC_ResolIR.f90: length of coef must be an odd value"
   RETURN
 endif
 !--Case of only one coef and this coef is not equal to 1 => reject filter
 coefsum=sum(coef)
 if ( ncoef==1 .and. (coefsum>(1+eps) .or. coefsum<(1-eps)) ) then 
   print*,"[ERROR] NDACC_ResolIR.f90: size of coef is equal to 1 but sum(coef) is not equal to 1"
   RETURN
 else
   if ( ncoef==1 .and. (coefsum>=(1-eps) .and. coefsum<=(1+eps)) ) then
     !--Case of no filtering
     status=1 !low pass filter
     !print*,"[DEBUG] No filtering case"
   else
     !--Common case: at least 3 coefs, and number of coefs is odd
     nc=(ncoef-1)/2     !number of points before and after the central point / central point = nc+1
     do i=1,nc
       if (abs(coef(nc+1-i)-coef(nc+1+i)) <= abs(coef(nc+1-i))*1e-3) then
         status = 1     !coef(nc+1-i) = coef(nc+1+i) (symmetric) =>  Smoothing filter
       else
         equalto=.false.
         if (coef(nc+1)<=(0.+eps) .and. coef(nc+1)>=(0.-eps)) then
           equalto=.true.
         endif
         if ( ( abs(coef(nc+1-i)+coef(nc+1+i)) <= abs(coef(nc+1-i))*1e-3 ) .and. equalto ) then
           status = 2   !coef(nc+1-i) = - coef(nc+1+i) and coef(nc+1)=0 (anti-symmetric) =>  Derivative filter (status=2)
         else
           status = 0
           print*,"[ERROR] NDACC_ResolIR.f90: bad coef values: neither symmetric or anti-symmetric"
           RETURN
         endif
       endif 
     enddo
   endif
 endif

 !--If first call to NDACC_ResolIR: initialize input signal (Dirac at central point)
 !----------------------------------------------------------------------------------
 if (firstcall) then
   irinp(nk/2+1)=1.
 endif

 !--If derivative filter: integrate input signal (Dirac becomes Heavyside step at nk/2+1)
 !---------------------------------------------------------------------------------------
 if (status==2) then 
  irtemp(1)=irinp(1)
  do k=2,nk
    irtemp(k) = sum(irinp(1:k)) 
  enddo
  irinp=irtemp
 endif
 irmax=maxval(irinp)
 if (irmax<=(0+eps) .and. irmax>=(0.-eps)) then
   status=0
   print*,"[ERROR] NDACC_ResolIR.f90: integration error"
   RETURN
 endif
 irinp=irinp/irmax

 !--Apply filter (calculate output impulse response irout)
 !--------------------------------------------------------
 !--Convolution
 do k=(nc+1),(nk-nc)
   irout(k)=0.
   j=1
   do i=(k-nc),(k+nc)
     irout(k)=irout(k)+irinp(i)*coef(j)
     j=j+1
   enddo
 enddo
 !--Retrieve irout max value and position 
 irmax=maxval(irout)
 if (irmax<=(0+eps) .and. irmax>=(0.-eps)) then
   status=0
   print*,"[ERROR] NDACC_ResolIR.f90: convolution error"
   RETURN
 endif
 kmax=maxloc(irout)
 if (kmax(1)<=1 .or. kmax(1)>=nk) then
   status=0
   print*,"[ERROR] NDACC_ResolIR.f90: output signal maximum invalid position"
   RETURN
 endif
 !--Standardize irout at 1
 irout=irout/irmax

 !--Identify FWHM , i.e. where irout=0.5*irmax --> x1 and x2
 !----------------------------------------------------------
 !--irtemp: subsract 0.5
 irtemp=irout-0.5 
 !--First zero crossing (before max point)
 fwhmfound=.false.
 do k=kmax(1),2,-1
   if ( sign(irtemp(k),irtemp(k-1)) <= 0. ) then  !if irtemp(k-1)>=0 -> sign=a else sign=-a
     x1=k-irtemp(k)/(irtemp(k)-irtemp(k-1))
     fwhmfound=.true.
     exit    !stop loop
   endif
 enddo  
 if (.not. fwhmfound) then
     status=0
     print*,"[ERROR] NDACC_ResolIR.f90: fwhm not found"
     RETURN
 endif
 !--Second zero crossing (after max point)
 fwhmfound=.false.
 do k=kmax(1),(nk-1)
   if ( sign(irtemp(k),irtemp(k+1)) <= 0. ) then 
     x2=k+irtemp(k)/(irtemp(k)-irtemp(k+1))
     fwhmfound=.true.
     exit    !stop loop
   endif
 enddo  
 if (.not. fwhmfound) then
     status=0
     print*,"[ERROR] NDACC_ResolIR.f90: fwhm not found"
     RETURN
 endif

 !--Calculate dzfwhm
 dzfwhm = dzsampling*(x2-x1)

 RETURN

 END SUBROUTINE NDACC_ResolIR
